@@ -1,5 +1,5 @@
 /*
- *   Copyright (C) 1997, 1998, 1999 Loic Dachary
+ *   Copyright (C) 1997, 1998, 1999, 2000 Loic Dachary
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -59,15 +59,16 @@
 #include <ftp.h>
 #include <mime.h>
 #include <dirsel.h>
+#include <server.h>
+#include <html_content.h>
 #ifdef LANGREC
 #include <decodage_usr.h>
 #include <dicoMySQL_def.h>
-#include <html_content.h>
 #include <html.h>
 #include <write_file.h>
 #endif /* LANGREC */
-#include <WebbaseDl.h>
-#include <WebbaseHook.h>
+#include <webbasedl.h>
+#include <webbase_hook.h>
 
 static int verbose = 1;
 
@@ -114,8 +115,6 @@ static struct option long_options[] =
   {"quiet_crawl", 0, &verbose, 0},
   /*- Verbose allow/disallow handling */
   {"verbose_dirsel", 0, 0, 0},
-  /*- Do not call inverted index hooks */
-  {"no_hook", 0, 0, 0},
   /*- <name> Use <name> inverted index library (default mifluz) */
   {"hook", 1, 0, 0},
   /*- <bytes> Size in bytes of the longest url (default 255) */
@@ -179,7 +178,6 @@ static struct option_help long_options_help[] =
   {"verbose_crawl","medium verbose crawl" },
   {"quiet_crawl", "silent crawl"},
   {"verbose_dirsel", "verbose allow/disallow handling"},
-  {"no_hook", "do not call inverted index hooks"},
   {"hook <name>", "use <name> inverted index library (default mifluz)"},
   {"url_max_size <bytes>", "size in bytes of the longest url (default 255)"},
   {"size_hrefs <bytes>", "maximum size in bytes of url buffer (default 100K)"},
@@ -245,14 +243,9 @@ crawl_params_t* crawl_init(int argc, char** argv)
   return crawl_alloc(argc, argv, long_options);
 }
 
-#ifdef HAVE_MIFLUZ_H
-extern "C" {
-void* hooksmifluz_init();
-}
-#endif /* HAVE_MIFLUZ_H */
-
 crawl_params_t* crawl_alloc(int argc, char** argv, struct option options[])
 {
+  webbasedl_init_function_t hook_init = 0;
   crawl_params_t* params = params_alloc();
   webbase_url_start_t* start = &params->start_in_core;
   params->agent = strdup(CRAWL_USER_AGENT);
@@ -348,22 +341,19 @@ crawl_params_t* crawl_alloc(int argc, char** argv, struct option options[])
         S(regex_allow,regex_allow_length,WEBBASE_URL_START_REGEX_ALLOW);
         S(regex_disallow,regex_disallow_length,WEBBASE_URL_START_REGEX_DISALLOW);
 #undef S
-	} else if(!strcmp(options[option_index].name, "no_hook")) {
-	  params->no_hook = 1;
-	} else if(!strcmp(options[option_index].name, "hook")) {
+	} else if(!strcmp(options[option_index].name, "agent")) {
 	  free(params->agent);
 	  params->agent = strdup(optarg);
 	} else if(!strcmp(options[option_index].name, "hook")) {
 	  /*
 	   * Load the inverted index library 
 	   */
-	  webbasedl_init_function_t function = WebbaseDl::Instance()->Find(optarg);
-	  if(!function)
+	  hook_init = webbasedl(optarg);
+	  if(!hook_init) {
 	    fprintf(stderr, "crawl::crawl_alloc: cannot load hook %s\n", optarg);
-	  else 
-	    if (verbose)
-	      fprintf(stderr,"crawl::crawl_alloc: initializing hook");
-	    params->hook = (WebbaseHook*)(*function)();
+	  } else {
+	    if (verbose > 1) fprintf(stderr,"crawl::crawl_alloc: found init function for hook %s\n", optarg);
+	  }
 	  break;
 	}
 	found = 0;
@@ -410,17 +400,13 @@ crawl_params_t* crawl_alloc(int argc, char** argv, struct option options[])
   /*
    * Create the inverted index module
    */
-#ifdef HAVE_MIFLUZ_H
-  if(!params->no_hook) {
-    params->hook = (WebbaseHook*)hooksmifluz_init();
-    params->hook->Parse(argc, argv);
-    params->hook->Init(params->base);
-  } else {
-#endif /* HAVE_MIFLUZ_H */
-    params->hook = new WebbaseHook();
-#ifdef HAVE_MIFLUZ_H
+  if(hook_init) {
+    params->hook = (webbase_hook_params_t*)(*hook_init)();
+    params->hook->getopt(params->hook, argc, argv);
+    params->hook->init(params->hook, params->base);
+    server_hook(params->hook);
+    if(verbose) fprintf(stderr, "loaded hook library %s\n", params->hook->name ? params->hook->name : "(unknown)");
   }
-#endif /* HAVE_MIFLUZ_H */
   if(params->candidate_limit <= 0)
     params->candidate_limit = CRAWL_PARAMS_CANDIDATE_LIMIT;
 
@@ -435,24 +421,55 @@ static void crawl_rebuild_1(char* paramsp, webbase_url_t* webbase_url)
   crawl_params_t* params = (crawl_params_t*)paramsp;
 
   webbase_lock_ignore(params->base, webbase_url->w_rowid, "url");
-  if(params->hook->Ok(webbase_url)) {
+  if(params->hook->ok(params->hook, webbase_url)) {
     char query[256];
-#if 0
-    char* path = uri_furi_string(webbase_url->w_url, strlen(webbase_url->w_url), URI_FURI_REAL_PATH);
-#else
-    char* path = webbase_url_file(webbase_url);
-#endif
-    if(file_exists(path)) {
-      params->hook->Server(robots_url2server_id(params->base, webbase_url->w_url));
-      params->hook->Insert(webbase_url);
-      if(verbose > 1) fprintf(stderr, "\trebuild %s (%d)\n", webbase_url->w_url, webbase_url->w_hookid);
-      sprintf(query, "update url set hookid = %d where rowid = %d\n", webbase_url->w_hookid, webbase_url->w_rowid);
-      smysql_query(&params->base->mysql, query);
-      params->hook->InsertWatch();
-    } else {
-      if(verbose > 1) fprintf(stderr, "\tmissing file for %s\n", webbase_url->w_url);
-      sprintf(query, "update url set mtime = 0 where rowid = %d\n", webbase_url->w_rowid);
+#ifdef WEBBASE_CONTENT_BASE
+    MYSQL_RES *res;
+    MYSQL_ROW row;
+
+    webbase_url->content_length = 0;
+
+#ifdef TABLE_SPLIT
+    sprintf(query, "select content from url_content%02d where rowid = %d", (webbase_url->w_rowid % TABLE_SPLIT_SIZE), webbase_url->w_rowid);
+#else /* TABLE_SPLIT */
+    sprintf(query, "select content from url_content where rowid = %d", webbase_url->w_rowid);
+#endif /* TABLE_SPLIT */
+    smysql_query(&params->base->mysql, query);
+    res = smysql_store_result(&params->base->mysql);
+
+    if(mysql_num_rows(res) > 0) {
+      if((row = mysql_fetch_row(res))) {
+	unsigned long* lengths = mysql_fetch_lengths(res);
+	if(row[0]) {
+	  webbase_url->content = row[0];
+	  webbase_url->content_length = lengths[0];
+	  webbase_url->content_size = 0;
+	}
+      }
     }
+    
+    if(webbase_url->content_length > 0)
+#else /* WEBBASE_CONTENT_BASE */
+    char* path = webbase_url_file(webbase_url->w_rowid);
+    if(file_exists(path))
+#endif /* WEBBASE_CONTENT_BASE */
+      {
+	params->hook->server(params->hook, server_id(params->base, webbase_url->w_url));
+	params->hook->insert(params->hook, webbase_url);
+	if(verbose > 1) fprintf(stderr, "\trebuild %s (%d)\n", webbase_url->w_url, webbase_url->w_hookid);
+	sprintf(query, "update url set hookid = %d where rowid = %d\n", webbase_url->w_hookid, webbase_url->w_rowid);
+	smysql_query(&params->base->mysql, query);
+	params->hook->insert_watch(params->hook);
+      } else {
+	if(verbose > 1) fprintf(stderr, "\tmissing url content for %s\n", webbase_url->w_url);
+	sprintf(query, "update url set mtime = 0 where rowid = %d\n", webbase_url->w_rowid);
+      }
+#ifdef WEBBASE_CONTENT_BASE
+    webbase_url->content = 0;
+    webbase_url->content_length = 0;
+    mysql_free_result(res);
+#endif /* WEBBASE_CONTENT_BASE */
+
   }
   webbase_lock_ignore(params->base, 0, 0);
   if(verbose == 1) {
@@ -472,7 +489,7 @@ static void crawl_rebuild_1(char* paramsp, webbase_url_t* webbase_url)
  * Re-index everything or just of a part of the documents if the -where_url option
  * is specified
  */
-void crawl_rebuild(crawl_params_t* params, int resume, char *where)
+void crawl_rebuild(crawl_params_t* params, int resume, char *wherep)
 {
   if(resume > 1) {
     fprintf(stderr, "resuming rebuild\n");
@@ -480,30 +497,30 @@ void crawl_rebuild(crawl_params_t* params, int resume, char *where)
     if (verbose > 1)
       fprintf(stderr,"starting crawl_rebuild\n");
 
-    if (params->no_hook)
-      {
-	fprintf(stderr,"rebuild functionnality require hooks library");
-	exit(-1);
-      }
+    if (!params->hook) {
+      fprintf(stderr,"rebuild require hooks library\n");
+      exit(-1);
+    }
     
-    params->hook->RebuildInit();
+    params->hook->rebuild_init(params->hook, wherep);
   }
     
-  params->hook->RebuildStart();
+  params->hook->rebuild_start(params->hook);
 
   /*
    * compose where clause
    */
-  char* regexp_where = smalloc(128 + (where ? strlen(where) : 0));
-  if(where && where[0] != '\0') {
-    sprintf(regexp_where, "where %s and hookid = 0", where);
+  char* where = smalloc(128 + (wherep ? strlen(wherep) : 0));
+  if(wherep && wherep[0] != '\0') {
+    sprintf(where, "where ( %s ) and hookid = 0", wherep);
   } else {
-    sprintf(regexp_where, "where hookid = 0");
+    sprintf(where, "where hookid = 0");
   }
 
-  webbase_walk_url(params->base, regexp_where, crawl_rebuild_1, (char*)params, WEBBASE_GET_URL_LIGHT);
+  webbase_walk_url(params->base, where, crawl_rebuild_1, (char*)params, WEBBASE_GET_URL_LIGHT);
 
-  params->hook->RebuildEnd();
+  if(params->hook)
+    params->hook->rebuild_end(params->hook);
 }
 
 /*
@@ -524,6 +541,7 @@ void crawl_urls(crawl_params_t* params, char* urls)
 {
   char* sql = smalloc(strlen(urls) + 128);
   sprintf(sql, "select url from url where %s", urls);
+  if(verbose) fprintf(stderr, "crawl_urls: %s\n", sql);
   sql_select(&params->base->mysql, 0, sql, crawl_urls_1, (char*)params);
   free(sql);
 }
@@ -537,9 +555,9 @@ static void crawl_rehook_1(char* paramsp, webbase_url_t* webbase_url)
   crawl_params_t* params = (crawl_params_t*)paramsp;
 
   webbase_lock_ignore(params->base, webbase_url->w_rowid, "url");
-  if(params->hook->Ok(webbase_url)) {
+  if(params->hook->ok(params->hook, webbase_url)) {
     char query[256];
-    params->hook->Insert(webbase_url);
+    params->hook->insert(params->hook, webbase_url);
     if(verbose > 1) fprintf(stderr, "\trehook %s (%d)\n", webbase_url->w_url, webbase_url->w_hookid);
     sprintf(query, "update url set hookid = %d where rowid = %d\n", webbase_url->w_hookid, webbase_url->w_rowid);
     smysql_query(&params->base->mysql, query);
@@ -570,7 +588,7 @@ void crawl_rehook(crawl_params_t* params)
    * and remove from the hooked database records than
    * should not be there.
    */
-  params->hook->RehookStart();
+  params->hook->rehook_start(params->hook);
   /*
    * Insert in hooked database all records that must
    * be hooked.
@@ -585,7 +603,7 @@ void crawl_rehook(crawl_params_t* params)
   /*
    * Close the hooked database (index validation etc..)
    */
-  params->hook->RehookEnd();
+  params->hook->rehook_end(params->hook);
 }
 
 /*
@@ -827,6 +845,7 @@ void hp_unload(crawl_params_t* params, char* url, int keep_start)
       sprintf(query, "update start set info = 0, count = 0 where rowid = %d", start.rowid);
     } else {
       sprintf(query, "delete from start where rowid = %d", start.rowid);
+      server_remove(params->base, url);
     }
     smysql_query(&base->mysql, query);
   }
@@ -866,7 +885,11 @@ void hp_show(crawl_params_t* params, char* url, char* fields, char* where)
   str2md5ascii(url, url_length, url_md5);
 
   if(fields == 0 || strlen(fields) == 0) {
+#ifdef WEBBASE_CONTENT_BASE
+    fields = "start.*,url.*,url_complete.*,url_content.*";
+#else /* WEBBASE_CONTENT_BASE */
     fields = "start.*,url.*,url_complete.*";
+#endif /* WEBBASE_CONTENT_BASE */
   }
 
   if(where == 0) {
@@ -878,7 +901,11 @@ void hp_show(crawl_params_t* params, char* url, char* fields, char* where)
     where = tmp_where;
   }
   
+#ifdef WEBBASE_CONTENT_BASE
+  templ = "select %s from start left join start2url on start.rowid = start2url.start left join url on start2url.url = url.rowid left join url_complete on url.complete_rowid = url_complete.rowid left join url_content on url.rowid = url_content.rowid where start.url_md5 = '%.*s' %s";
+#else /* WEBBASE_CONTENT_BASE */
   templ = "select %s from start left join start2url on start.rowid = start2url.start left join url on start2url.url = url.rowid left join url_complete on url.complete_rowid = url_complete.rowid where start.url_md5 = '%.*s' %s";
+#endif /* WEBBASE_CONTENT_BASE */
   query = smalloc(strlen(templ) + strlen(fields) + strlen(where) + MD5_ASCII_SIZE + 1);
   
   sprintf(query, templ, fields, MD5_ASCII_SIZE, url_md5, where);
@@ -889,6 +916,119 @@ void hp_show(crawl_params_t* params, char* url, char* fields, char* where)
   free(where);
 }
 
+static int hp_show_indexable_1(int info, html_content_parsed_t* parsed, void* data)
+{
+  unsigned char* buffer = parsed->buffer0;
+  int buffer_length = parsed->buffer0_length;
+  
+  if(info == HTML_CONTENT_META) {
+      buffer = parsed->buffer1;
+      buffer_length = parsed->buffer1_length;
+
+      if(strncasecmp((const char*)parsed->buffer0, "keyword", 7) &&
+	 strncasecmp((const char*)parsed->buffer0, "description", 11)) {
+	buffer = 0;
+	buffer_length = 0;
+      }
+  }
+
+  if(buffer)  printf("%.*s", buffer_length, buffer);
+
+  return 1;
+}
+
+void hp_show_indexable(crawl_params_t* params, char* url)
+{
+  int url_length = strlen(url);
+  unsigned char url_md5[MD5_ASCII_SIZE];
+  char* query;
+  char* templ;
+  char rowid_string[WEBBASE_INTEGER_VALUE_SIZE];
+  rowid_string[0] = '\0';
+
+  if(url_length > WEBBASE_URL_LENGTH) {
+    fprintf(stderr, "hp_show_indexable: %s too long (ignored)\n", url);
+    return;
+  }
+
+  str2md5ascii(url, url_length, url_md5);
+
+  templ = "select rowid from url where url_md5 = '%.*s'";
+  query = smalloc(strlen(templ) + MD5_ASCII_SIZE + 1);
+  
+  sprintf(query, templ, MD5_ASCII_SIZE, url_md5);
+
+  sql_select_value(&params->base->mysql, query, rowid_string, WEBBASE_INTEGER_VALUE_SIZE);
+
+  free(query);
+  
+  {
+    html_content_t walk;
+#ifdef WEBBASE_CONTENT_BASE
+    char query[256];
+    char* content = 0;
+    int content_length = 0;
+    MYSQL_RES *res;
+    MYSQL_ROW row;
+    int rowid = atoi(rowid_string);
+
+    content_length = 0;
+
+#ifdef TABLE_SPLIT
+    sprintf(query, "select content from url_content%02d where rowid = %d", (rowid % TABLE_SPLIT_SIZE), rowid );
+#else /* TABLE_SPLIT */
+    sprintf(query, "select content from url_content where rowid = %d", rowid);
+#endif /* TABLE_SPLIT */
+    smysql_query(&params->base->mysql, query);
+    res = smysql_store_result(&params->base->mysql);
+
+    if(mysql_num_rows(res) > 0) {
+      if((row = mysql_fetch_row(res))) {
+	unsigned long* lengths = mysql_fetch_lengths(res);
+	if(row[0]) {
+	  content = row[0];
+	  content_length = lengths[0];
+	}
+      }
+    }
+    
+    if(content_length <= 0) {
+      fprintf(stderr, "hp_show_indexable: contains nothing\n");
+      mysql_free_result(res);
+      return;
+    }
+    
+#else /* WEBBASE_CONTENT_BASE */
+    char* path = webbase_url_file(atoi(rowid_string));
+    if(verbose > 1) printf("%s\n", path);
+
+#endif /* WEBBASE_CONTENT_BASE */
+
+    // fill the html_content_t structure
+    html_content_reset(&walk);
+#ifdef WEBBASE_CONTENT_BASE
+    walk.parser.info = HTML_SOURCE_STRING;
+    // path to a cached file
+    walk.parser.source = content;
+    walk.parser.source_length = content_length;
+#else /* WEBBASE_CONTENT_BASE */
+    walk.parser.info = HTML_SOURCE_FILENAME;
+    // path to a cached file
+    walk.parser.source = path;
+    walk.parser.source_length = strlen(path);
+#endif /* WEBBASE_CONTENT_BASE */
+    // callback called for each text chunk found
+    walk.content_callback = hp_show_indexable_1;
+    // parse the data stored in cache file
+    if (!html_content_parse(&walk)) {
+      fprintf(stderr, "hp_show_indexable: cannot convert to text %s\n", url);
+    }
+#ifdef WEBBASE_CONTENT_BASE
+    mysql_free_result(res);
+#endif /* WEBBASE_CONTENT_BASE */
+  }
+}
+
 /*
  * Explore or update all the starting points
  */
@@ -998,7 +1138,7 @@ static void hp_load_mirror(char* paramsp, webbase_url_start_t* start)
    */
   context.dirsel_key = strdup(dirsel_key(context.home));
 
-  params->hook->InfoSet(start->hook_info);
+  if(params->hook) params->hook->info_set(params->hook, start->hook_info);
 
   switch(start->info & WEBBASE_URL_START_STATE_MASK) {
   case WEBBASE_URL_START_VIRGIN:
@@ -1147,10 +1287,18 @@ static void mirror_cleanup(crawl_context_t* context)
       int hookid = row[2] ? atoi(row[2]) : 0;
       char* url = row[3];
 
-      if(hookid && !context->params->no_hook)
-	context->params->hook->DeleteId(hookid, url, strlen(url));
+      if(hookid && context->params->hook)
+	context->params->hook->delete_id(context->params->hook, hookid, url, strlen(url));
       sprintf(query, "delete from url where rowid = %d", rowid);
       smysql_query(&base->mysql, query);    
+#ifdef WEBBASE_CONTENT_BASE
+#ifdef TABLE_SPLIT
+      sprintf(query, "delete from url_content%02d where rowid = %d", (rowid % TABLE_SPLIT_SIZE), rowid);
+#else /* TABLE_SPLIT */
+      sprintf(query, "delete from url_content where rowid = %d", rowid);
+#endif /* TABLE_SPLIT */
+      smysql_query(&base->mysql, query);    
+#endif /* WEBBASE_CONTENT_BASE */
       if(complete_rowid) {
 	sprintf(query, "delete from url_complete where rowid = %d", complete_rowid);
 	smysql_query(&base->mysql, query);    
@@ -1416,7 +1564,7 @@ static int mirror_query(crawl_context_t* context, char* query)
       char* url = row[0];
 
       robots_load(context->params->robots, context, url);
-      context->params->hook->Server(robots_server_id(context->params->robots));
+      if(context->params->hook) context->params->hook->server(context->params->hook, server_id(context->params->base, url));
       if((webbase_url = mirror(context, url))) {
 	/*
 	 * walk_href returns the status of the last 'mirror_crawl_continue'
@@ -1566,10 +1714,10 @@ static webbase_url_t* mirror_1(crawl_context_t* context, char* url)
   webbase_url_t* webbase_url = mirror_2(context, url);
 
   if(webbase_url) {
-    if(!context->params->no_hook) {
+    if(context->params->hook) {
       char query[256];
       int hook_inserted = 0;
-      int hook_insert_now = context->params->hook->Ok(webbase_url);
+      int hook_insert_now = context->params->hook->ok(context->params->hook, webbase_url);
       int do_update = 0;
 
       /*
@@ -1585,12 +1733,12 @@ static webbase_url_t* mirror_1(crawl_context_t* context, char* url)
        */
       webbase_lock_ignore(context->base, webbase_url->w_rowid, "url");
       if(hook_inserted && hook_insert_now) 
-	context->params->hook->Update(webbase_url);
+	context->params->hook->update(context->params->hook, webbase_url);
       else if(hook_inserted && !hook_insert_now) {
-	context->params->hook->Delete(webbase_url);
+	context->params->hook->tdelete(context->params->hook, webbase_url);
 	do_update = 1;
       } else if(!hook_inserted && hook_insert_now) {
-	context->params->hook->Insert(webbase_url);
+	context->params->hook->insert(context->params->hook, webbase_url);
 	do_update = 1;
       } else
 	;
@@ -1679,11 +1827,19 @@ static webbase_url_t* mirror_2(crawl_context_t* context, char* url)
    * Force crawling if the document is not in the cache.
    */
   if(context->touch && (webbase_url->w_info & WEBBASE_URL_INFO_CONTENT)) {
-#if 0
-    char* path = uri_furi_string(webbase_url->w_url, strlen(webbase_url->w_url), URI_FURI_REAL_PATH);
-#else
-    char* path = webbase_url_file(webbase_url);
-#endif
+#ifdef WEBBASE_CONTENT_BASE
+    char query[128];
+#ifdef TABLE_SPLIT
+    sprintf(query, "select rowid from url_content%02d where rowid = %d", (webbase_url->w_rowid % TABLE_SPLIT_SIZE), webbase_url->w_rowid);
+#else /* TABLE_SPLIT */
+    sprintf(query, "select rowid from url_content where rowid = %d", webbase_url->w_rowid);
+#endif /* TABLE_SPLIT */
+    if(!sql_select_exists(&context->base->mysql, query)) {
+      if(verbose > 1) fprintf(stderr, "\tmirror: touching\n");
+      webbase_url->w_crawl = time(0) - WEBBASE_TIME_DAY;
+    }
+#else /* WEBBASE_CONTENT_BASE */
+    char* path = webbase_url_file(webbase_url->w_rowid);
     if(path) {
       if(!file_exists(path)) {
 	if(verbose > 1) fprintf(stderr, "\tmirror: touching\n");
@@ -1692,6 +1848,7 @@ static webbase_url_t* mirror_2(crawl_context_t* context, char* url)
     } else {
       fprintf(stderr, "cannot uncache %s, unable to resolve path (ignored)\n", webbase_url->w_url);
     }
+#endif /* WEBBASE_CONTENT_BASE */
   }
 
   /*
@@ -1974,11 +2131,7 @@ static int mirror_ftp(crawl_context_t* context, webbase_url_t* webbase_url, uri_
 
     {
 	int isdir;
-#if 0
-	char* path = uri_furi_string(webbase_url->w_url, strlen(webbase_url->w_url), URI_FURI_REAL_PATH);
-#else
-	char* path = webbase_url_file(webbase_url);
-#endif
+	char* path = webbase_url_file(webbase_url->w_rowid);
 	int code;
 
 	if(path) {
@@ -2026,28 +2179,6 @@ static int mirror_ftp(crawl_context_t* context, webbase_url_t* webbase_url, uri_
     return 0;
 }
 
-#ifdef LANGREC
-static int html_content_collect_begin(int info, html_content_parsed_t* parsed, void* data)
-{
-  int ret = 1;
-
-  switch(info) {
-
-  case HTML_CONTENT_META:
-  case HTML_CONTENT_TITLE:
-    break;
-
-  case HTML_CONTENT_TEXT:
-    {
-      append_file((char *)data,(char *)parsed->buffer0,parsed->buffer0_length);
-    }
-    break;
-  }
-
-  return ret;
-}
-#endif
-
 /*
  * Crawl a single URL using HTTP
  */
@@ -2059,6 +2190,7 @@ static int mirror_http(crawl_context_t* context, webbase_url_t* webbase_url, uri
   int sd;
   int error = 0;
   get = mirror_request_http(context, webbase_url, url_object);
+  http_context.crawl = context;
   http_context.webbase_url = webbase_url;
   http_context.cookies = context->params->cookies;
   http->callback_arg = (char*)&http_context;
@@ -2093,7 +2225,7 @@ static int mirror_http(crawl_context_t* context, webbase_url_t* webbase_url, uri
 	 (webbase_url->w_info & WEBBASE_URL_INFO_COOKIE) &&
 	 (webbase_url->w_code == WEBBASE_URL_CODE_MOVED_PERMANENTLY ||
 	  webbase_url->w_code == WEBBASE_URL_CODE_MOVED_TEMPORARILY)) {
-	if(verbose) fprintf(stderr, "\tmirror_2: cookie loop\n");
+	if(verbose > 1) fprintf(stderr, "\tmirror_2: cookie loop\n");
 	webtools_close(http, sd);
 	cookie_loop = 1;
 	return mirror_http(context, webbase_url, url_object, url);
@@ -2105,15 +2237,31 @@ static int mirror_http(crawl_context_t* context, webbase_url_t* webbase_url, uri
 	if((error = webtools_reader(http, sd)) == WEBTOOLS_READER_OK ||
 	   error == WEBTOOLS_READER_TRUNC) {
 	  if(verbose > 1) fprintf(stderr, "\tloading %s : body read\n", url);
-	  webbase_url_hrefs_set(webbase_url, webbase_url->path, context->start->size_hrefs);
+	  webbase_url_hrefs_set(webbase_url, context->start->size_hrefs);
 
 #ifdef LANGREC
 	  {
 	    // language recognition
-	    // added by benoit orihuela (benoit.orihuela@IDEALX.com), on 2000/07/20
 	    ///////////////////////
 	    if (verbose>1) fprintf(stderr,"\tbeginning language recognition\n");
 
+#ifdef WEBBASE_CONTENT_BASE
+	    char language[DECODAGE_TAILLE_MAX];
+	    if(webbase_url->content_length > 0) {
+#ifdef DEFAULT_TRIE_PATH
+	      textlang_in_string(webbase_url->content, language, DEFAULT_TRIE_PATH);
+#else
+	      textlang_in_string(webbase_url->content, language, NULL);
+#endif
+	    }
+	    if (verbose>1) fprintf(stderr,"\tlanguage found : %s\n",language);
+
+	    if (language[0] != '\0')
+	      strncpy(webbase_url->w_language, language, WEBBASE_LANGUAGE_LENGTH);
+	    else
+	      strncpy(webbase_url->w_language, "unknown", WEBBASE_LANGUAGE_LENGTH);
+	    webbase_url->w_language[WEBBASE_LANGUAGE_LENGTH] = '\0';
+#else /* WEBBASE_CONTENT_BASE */
 	    char* raw_path = webbase_url->path;
 	    html_content_t walk;
 
@@ -2121,7 +2269,7 @@ static int mirror_http(crawl_context_t* context, webbase_url_t* webbase_url, uri
 	    char* f_lang = smalloc(strlen(raw_path) + 64);
 	    sprintf(f_lang, "%s.langrec", raw_path);
 	    if(mknod(f_lang, 0777, S_IFREG) < 0) {
-	      fprintf(stderr, "creat(%s) failed:", f_lang);
+	      fprintf(stderr, "mknod(%s) failed:", f_lang);
 	      perror("");
 	      exit(1);
 	    }
@@ -2132,7 +2280,7 @@ static int mirror_http(crawl_context_t* context, webbase_url_t* webbase_url, uri
 	    // path to a cached file
 	    walk.parser.source = raw_path;
 	    walk.parser.source_length = strlen(raw_path);
-	    // callback called for each href found
+	    // callback called for each text chunk found
 	    walk.content_callback = html_content_collect_begin;
 	    // path to the temporay file where content text is written
 	    walk.content_data = (void *)f_lang;
@@ -2140,29 +2288,33 @@ static int mirror_http(crawl_context_t* context, webbase_url_t* webbase_url, uri
 	    if (!html_content_parse(&walk)) {
   	      fprintf(stderr, "mirror_http: cannot convert to text %s\n", raw_path);
   	    } else {
-	    char language[DECODAGE_TAILLE_MAX];
-	    // find language in temp file (module : langrec / file : textlang_in_file)
+	      char language[DECODAGE_TAILLE_MAX];
+	      // find language in temp file (module : langrec / file : textlang_in_file)
 #ifdef DEFAULT_TRIE_PATH
-	    textlang_in_file(f_lang, language, DEFAULT_TRIE_PATH);
+	      textlang_in_file(f_lang, language, DEFAULT_TRIE_PATH);
 #else
-	    textlang_in_file(f_lang, language, NULL);
+	      textlang_in_file(f_lang, language, NULL);
 #endif
-	    if (verbose>1) fprintf(stderr,"\tlanguage found : %s\n",language);
+	      if (verbose>1) fprintf(stderr,"\tlanguage found : %s\n",language);
 
-	    if (language[0] != '\0')
-	      strncpy(webbase_url->w_language, language, WEBBASE_LANGUAGE_LENGTH);
-	    else
-	      strncpy(webbase_url->w_language, "unknown", WEBBASE_LANGUAGE_LENGTH);
-	    webbase_url->w_language[WEBBASE_LANGUAGE_LENGTH] = '\0';
-	    // erase temporary file
-	    rm(f_lang);
-	    free(f_lang);
+	      if (language[0] != '\0')
+		strncpy(webbase_url->w_language, language, WEBBASE_LANGUAGE_LENGTH);
+	      else
+		strncpy(webbase_url->w_language, "unknown", WEBBASE_LANGUAGE_LENGTH);
+	      webbase_url->w_language[WEBBASE_LANGUAGE_LENGTH] = '\0';
+	      // erase temporary file
+	      rm(f_lang);
+	      free(f_lang);
 	    }
+#endif /* WEBBASE_CONTENT_BASE */
   	  }
 #endif /* LANGREC */
-	  webbase_url_content_length_fix(webbase_url, webbase_url->path);
-	  webbase_url_mtime_fix(webbase_url);
+	  webbase_url_content_length_fix(webbase_url);
+#ifdef WEBBASE_CONTENT_BASE
+#else /* WEBBASE_CONTENT_BASE */
 	  webbase_url->path = 0;
+#endif /* WEBBASE_CONTENT_BASE */
+	  webbase_url_mtime_fix(webbase_url);
 	  webbase_url->w_info &= ~(WEBBASE_URL_INFO_TRUNCATED | WEBBASE_URL_INFO_READING);
 	  if(error == WEBTOOLS_READER_TRUNC)
 	    error = E2BIG;
@@ -2173,11 +2325,16 @@ static int mirror_http(crawl_context_t* context, webbase_url_t* webbase_url, uri
 	}
       } else if(!(webbase_url->w_info & WEBBASE_URL_INFO_CONTENT) ||
 		webbase_url->w_content_length <= 0) {
-#if 0
-	char* path = uri_furi_string(webbase_url->w_url, strlen(webbase_url->w_url), URI_FURI_REAL_PATH);
-#else
-	char* path = webbase_url_file(webbase_url);
-#endif
+#ifdef WEBBASE_CONTENT_BASE
+	char query[128];
+#ifdef TABLE_SPLIT
+	sprintf(query, "delete from url_content%02d where rowid = %d", (webbase_url->w_rowid % TABLE_SPLIT_SIZE), webbase_url->w_rowid);
+#else /* TABLE_SPLIT */
+	sprintf(query, "delete from url_content where rowid = %d", webbase_url->w_rowid);
+#endif /* TABLE_SPLIT */
+	smysql_query(&context->base->mysql, query);
+#else /* WEBBASE_CONTENT_BASE */
+	char* path = webbase_url_file(webbase_url->w_rowid);
 	if(path) {
 	  if(file_exists(path))
 	    {
@@ -2186,6 +2343,7 @@ static int mirror_http(crawl_context_t* context, webbase_url_t* webbase_url, uri
 	} else {
 	  fprintf(stderr, "cannot uncache %s, unable to resolve path (ignored)\n", webbase_url->w_url);
 	}
+#endif /* WEBBASE_CONTENT_BASE */
       }
     } else {
       error = errno;
@@ -2376,8 +2534,8 @@ static void mirror_schedule(crawl_context_t* context, webbase_url_t* webbase_url
  */
 void crawl_free(crawl_params_t* params)
 {
-  if(!params->no_hook)
-    params->hook->End();
+  if(params->hook)
+    params->hook->tend(params->hook);
   dirsel_end();
   robots_free(params->robots);
   cookies_free(params->cookies);
