@@ -414,13 +414,8 @@ static int dirsel_match(dirsel_t* entry, char* comparable)
 {
   int i;
   for(i = 0; i < entry->prefix_length; i++) {
-    /*
-     * Systematicaly skip the leading /
-     * If, by accident, the prefix does not start with /, take it completely.
-     * If the prefix is empty, ignore it.
-     */
     char method = entry->prefix[i][0] == DIRSEL_BOL ? DIRSEL_BOL : DIRSEL_ANY;
-    char* prefix = (method == DIRSEL_BOL) ? entry->prefix[i] + 1 : entry->prefix[i];
+    char* prefix = entry->prefix[i];
     int length = strlen(prefix);
     if(verbose) fprintf(stderr, "\tdirsel_match: compare prefix %s (deduced from %s) with %s using method %s\n", prefix, entry->prefix[i], comparable, (method == DIRSEL_BOL ? "bol" : "any"));
     if(length > 0) {
