@@ -140,6 +140,27 @@ static int html_href_collect(int info, uri_t* uri, void* data)
   return ret;
 }
 
+#ifdef WEBBASE_CONTENT_BASE
+int html_href(char* content, int content_length, webbase_url_t* webbase_url, int size_limit)
+{
+  html_href_t walk;
+  html_href_data_t href_data;
+
+  html_href_reset(&walk);
+  walk.parser.info = HTML_SOURCE_STRING;
+  walk.parser.source = content;
+  walk.parser.source_length = content_length;
+  walk.parser.ignore = HTML_URI_IMG;
+  walk.href_callback = html_href_collect;
+  walk.href_data = (void*)&href_data;
+
+  memset((char*)&href_data, '\0', sizeof(html_href_data_t));
+  href_data.webbase_url = webbase_url;
+  href_data.size_limit = size_limit;
+
+  return html_href_parse(&walk);
+}
+#else /* WEBBASE_CONTENT_BASE */
 int html_href(char* path, webbase_url_t* webbase_url, int size_limit)
 {
   html_href_t walk;
@@ -159,6 +180,7 @@ int html_href(char* path, webbase_url_t* webbase_url, int size_limit)
 
   return html_href_parse(&walk);
 }
+#endif /* WEBBASE_CONTENT_BASE */
 
 /*
  * Extract meta information on HTML file and build an excerpt
