@@ -19,10 +19,12 @@
 #ifndef _http_h
 #define _http_h
 
+#include <crawl_private.h>
 #include <webbase_url.h>
 #include <cookies.h>
 
 typedef struct http_context {
+  crawl_context_t* crawl;
   webbase_url_t* webbase_url;
   cookies_t* cookies;
 } http_context_t;
