@@ -24,6 +24,10 @@
  * and return the list of Allow strings in <allowp> and the 
  * list of Disallow strings in <disallowp>.
  */
+#ifdef WEBBASE_CONTENT_BASE
+int robots_parse(char* content, int content_length, char** allowp, char** disallowp);
+#else /* WEBBASE_CONTENT_BASE */
 int robots_parse(char* path, char** allowp, char** disallowp);
+#endif /* WEBBASE_CONTENT_BASE */
 
 #endif /* _robots_parser_h */
