@@ -43,6 +43,7 @@
 #include <webbase_url.h>
 #include <sqlutil.h>
 #include <md5str.h>
+#include <server.h>
 
 static int verbose = 0;
 
@@ -249,10 +250,10 @@ void webbase_lock_ignore(webbase_t*, int varint, char* varchar)
 
   if(varchar) {
     sprintf(name, "ECILA_IGNORE=%d%s", varint, varchar);
-    if(verbose) fprintf(stderr, "\tset lock_ignored \n");
+    if(verbose > 1) fprintf(stderr, "\tset lock_ignored \n");
   } else {
     strcpy(name, "ECILA_IGNORE=");
-    if(verbose) fprintf(stderr, "\tunset lock_ignored \n");
+    if(verbose > 1) fprintf(stderr, "\tunset lock_ignored \n");
   }
 
   if(putenv(name) != 0) {
@@ -299,7 +300,7 @@ int webbase_lock(webbase_t* base, int varint, char* varchar)
 
   sprintf(query, "insert low_priority into locks values ('%.*s', '%d')", md5_length, md5, base->pid);
 
-  if(verbose) fprintf(stderr, "\tlocking %s ...", name);
+  if(verbose > 1) fprintf(stderr, "\tlocking %s ...", name);
   while(locked == 0) {
     loop_count++;
     if(loop_count > base->lock_max_loop) {
@@ -315,7 +316,7 @@ int webbase_lock(webbase_t* base, int varint, char* varchar)
       }
       if(verbose) fprintf(stderr, " check pid for %s ", name);
       sprintf(query2, "select high_priority pid from locks where name = '%.*s'", md5_length, md5);
-      if(sql_select_value(&base->mysql, query2, pid_string, 32) == 0) {
+      if(sql_select_value(&base->mysql, query2, pid_string, WEBBASE_INTEGER_VALUE_SIZE) == 0) {
 	/*
 	 * Lock is gone
 	 */
@@ -338,7 +339,7 @@ int webbase_lock(webbase_t* base, int varint, char* varchar)
 	}
       }
     } else {
-      if(verbose) fprintf(stderr, " locked\n");
+      if(verbose > 1) fprintf(stderr, " locked\n");
       locked = 1;
     }
   }
@@ -374,13 +375,13 @@ void webbase_unlock(webbase_t* base, int varint, char* varchar)
 
   sprintf(query, "delete low_priority from locks where name = '%.*s' and pid = %d", md5_length, md5, base->pid);
 
-  if(verbose) fprintf(stderr, "\tunlocking %s ...", name);
+  if(verbose > 1) fprintf(stderr, "\tunlocking %s ...", name);
   smysql_query(&base->mysql, query);
   if(mysql_affected_rows(&base->mysql) == 0) {
     char pid_string[32];
     if(verbose) fprintf(stderr, "not found ");
     sprintf(query, "select high_priority pid from locks where name = '%.*s'", md5_length, md5);
-    if(sql_select_value(&base->mysql, query, pid_string, 32) == 0) {
+    if(sql_select_value(&base->mysql, query, pid_string, WEBBASE_INTEGER_VALUE_SIZE) == 0) {
       if(verbose) fprintf(stderr, ", no row ");
     } else {
       if(verbose) fprintf(stderr, ", held by %s ", pid_string);
@@ -389,7 +390,7 @@ void webbase_unlock(webbase_t* base, int varint, char* varchar)
     abort();
     exit(1);
   }
-  if(verbose) fprintf(stderr, " done\n");
+  if(verbose > 1) fprintf(stderr, " done\n");
 }
 
 /*
@@ -455,6 +456,7 @@ void webbase_walk_url(webbase_t* base, char* where, webbase_walk_url_callback_t
 
   static_alloc(&query, &query_size, 256 + where_length);
   sprintf(query, "select rowid from url %s", where);
+  if(verbose) fprintf(stderr, "webbase_walk_url: %s\n", query);
 
   smysql_query(&base->mysql, query);
   res = smysql_store_result(&base->mysql);
@@ -646,6 +648,7 @@ void webbase_update_start(webbase_t* base, webbase_url_start_t* start)
     sprintf(query + strlen(query), " where rowid = %d", start->rowid);
     if(verbose > 5) fprintf(stderr, "update start: %s\n", query);
   } else {
+    server_add(base, start->url);
     sprintf(query, "replace into start ("WEBBASE_START_FIELDS_INSERT") values (%d, '%s', '%s', %d, ",
 	    start->rowid,
 	    sql_quote_char(&url, &url_size, start->url, strlen(start->url)),
