@@ -185,22 +185,22 @@ typedef struct webbase_url_common {
  */
 #define WEBBASE_URL_START_SLEEPY	 0x00001
 #define WEBBASE_URL_START_UNESCAPE	 0x00002
-#define WEBBASE_URL_START_NOCOOKIE	 0x00008
-#define WEBBASE_URL_START_ACCEPT	 0x00010
-#define WEBBASE_URL_START_HEURISTICS	 0x00020
-#define WEBBASE_URL_START_STICKY	 0x00040
-#define WEBBASE_URL_START_FILTER	 0x00080
-#define WEBBASE_URL_START_HOMEFREE	 0x00100
-#define WEBBASE_URL_START_VIRGIN	 0x00200
-#define WEBBASE_URL_START_EXPLORING	 0x00400
-#define WEBBASE_URL_START_EXPLORED	 0x00800
-#define WEBBASE_URL_START_UPDATING	 0x01000
-#define WEBBASE_URL_START_IN_CORE	 0x02000
-#define WEBBASE_URL_START_ALLOW		 0x04000
-#define WEBBASE_URL_START_DISALLOW	 0x08000
-#define WEBBASE_URL_START_HOOK_INFO	 0x10000
-#define WEBBASE_URL_START_REGEX_ALLOW	 0x20000
-#define WEBBASE_URL_START_REGEX_DISALLOW 0x40000
+#define WEBBASE_URL_START_NOCOOKIE	 0x00004
+#define WEBBASE_URL_START_ACCEPT	 0x00008
+#define WEBBASE_URL_START_HEURISTICS	 0x00010
+#define WEBBASE_URL_START_STICKY	 0x00020
+#define WEBBASE_URL_START_FILTER	 0x00040
+#define WEBBASE_URL_START_HOMEFREE	 0x00080
+#define WEBBASE_URL_START_VIRGIN	 0x00100
+#define WEBBASE_URL_START_EXPLORING	 0x00200
+#define WEBBASE_URL_START_EXPLORED	 0x00400
+#define WEBBASE_URL_START_UPDATING	 0x00800
+#define WEBBASE_URL_START_IN_CORE	 0x01000
+#define WEBBASE_URL_START_ALLOW		 0x02000
+#define WEBBASE_URL_START_DISALLOW	 0x04000
+#define WEBBASE_URL_START_HOOK_INFO	 0x08000
+#define WEBBASE_URL_START_REGEX_ALLOW	 0x10000
+#define WEBBASE_URL_START_REGEX_DISALLOW 0x20000
 
 #define WEBBASE_URL_START_STATE_MASK    0x1e00
 
@@ -211,8 +211,8 @@ typedef struct webbase_url_common {
 /*
  * Symbolic names of document parts
  */
-#define WEBBASE_URL_TAG_BODY		0x01
-#define WEBBASE_URL_TAG_TITLE		0x02
+#define WEBBASE_URL_TAG_TITLE		0x01
+#define WEBBASE_URL_TAG_BODY		0x02
 #define WEBBASE_URL_TAG_KEY		0x03
 #define WEBBASE_URL_TAG_DESCRIPTION	0x04
 
@@ -331,11 +331,17 @@ typedef struct webbase_url {
   int url_length;
   /* MD5 signature of the URL */
   unsigned char url_md5[MD5_ASCII_SIZE];
+#ifdef WEBBASE_CONTENT_BASE
+  char* content;
+  int content_size;
+  int content_length;
+#else /* WEBBASE_CONTENT_BASE */
   /*
    * Absolute path name of the file used to store the content of the
    * document.
    */
   char* path;
+#endif /* WEBBASE_CONTENT_BASE */
   webbase_url_complete_t data; 
 } webbase_url_t;
 
@@ -395,15 +401,15 @@ void webbase_url_location_set(webbase_url_t* webbase_url, char* location);
 /*
  * Check that expected document length match file size and fix if necessary
  */
-void webbase_url_content_length_fix(webbase_url_t* webbase_url, char* path);
+void webbase_url_content_length_fix(webbase_url_t* webbase_url);
 /*
  * Set content length from header
  */
 void webbase_url_content_length_set(webbase_url_t* webbase_url, int content_length);
 /*
- * Parse hrefs from <path> HTML file and store in url_complete fields.
+ * Parse hrefs from HTML content and store in url_complete fields.
  */
-void webbase_url_hrefs_set(webbase_url_t* webbase_url, char* path, int size_hrefs);
+void webbase_url_hrefs_set(webbase_url_t* webbase_url, int size_hrefs);
 /*
  * Print human readable representation of <webbase_url> 
  */
@@ -415,6 +421,6 @@ void webbase_url_mtime_fix(webbase_url_t* webbase_url);
 /*
  * Return file name for storage
  */
-char* webbase_url_file(webbase_url_t* webbase_url);
+char* webbase_url_file(int rowid);
 
 #endif /* _webbase_url_h */
