@@ -19,7 +19,15 @@
 #ifndef _find_h
 #define _find_h
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 typedef void (*find_callback)(char* path, char* dir, char* file);
 void find(char* dir, find_callback function);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* find_h */
