@@ -19,6 +19,14 @@
 #ifndef _mpath_h
 #define _mpath_h
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 int minimal_path(char* path, int path_length);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* mpath_h */
