@@ -19,6 +19,14 @@
 #ifndef _rm_h
 #define _rm_h
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 int rm(char* filename);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* rm_h */
