@@ -19,7 +19,15 @@
 #ifndef _write_file_h
 #define _write_file_h
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 void write_file(char* path, char* data, int size);
 void append_file(char* path, char* data, int size);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* write_file_h */
